# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This file is part of unity-gtk-module.

"""unity-gtk-module autopilot tests - top level package."""
